/* $LAAS: viamStruct.h 2010/10/27 11:51:52 mallet $
 *
 * Copyright (c) 2007-2010 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 *                                      Anthony Mallet on Mon Jan 22 2007
 */
#ifndef H_VIAMSTRUCT
#define H_VIAMSTRUCT

#include "viam/viamtypes.h"
#include "pomStruct.h"

#define VIAM_ID_MAX	64
#define VIAM_ID_EMPTY	"-"

typedef struct ViamId {
  char id[VIAM_ID_MAX];
} ViamId;

typedef enum ViamIO {
  VIAM_LOAD,
  VIAM_SAVE
} ViamIO;

typedef enum ViamActivate {
  VIAM_DISABLE,
  VIAM_ENABLE
} ViamActivate;


/* --- Bank and camera configuration  -------------------------------- */

typedef enum ViamImageUpdate {
  VIAM_SINGLE_BUFFERING = 1,
  VIAM_DOUBLE_BUFFERING = 2
} ViamImageUpdate;

typedef struct ViamBankCreate {
  ViamId		name;		/**< bank name */
  ViamImageUpdate	buffering;	/**< poster buffering */
  ViamActivate		tags;		/**< pom tagging */
} ViamBankCreate;

typedef struct ViamCameraCreate {
  ViamId	name;
  char		uid[1024];		/** uid specific to device */
} ViamCameraCreate;

typedef struct ViamBankAddCamera {
  ViamId	bank;
  ViamId	camera;
  ViamId	name;
} ViamBankAddCamera;


/* --- Image acquisition --------------------------------------------- */

typedef struct ViamHWModeList {
  int count;
  viam_hwmode_t modes[32 /* max number of modes - increase as needed */];
} ViamHWModeList;

typedef struct ViamHWMode {
  ViamId	camera;
  viam_hwmode_t mode;
} ViamHWMode;

typedef struct ViamAcquire {
  ViamId	bank;		/**< bank name */
  unsigned int	n;		/**< number of images */
} ViamAcquire;


/* --- Filters ------------------------------------------------------- */

typedef struct ViamFilterCap {
  viam_filtermethod_t methods[4 /* max number of modes - increase as needed */];
  viam_filterautomode_t automodes[6 /* max number of modes - increase as needed */];
} ViamFilterCap;

typedef struct ViamFilterGetCap {
  ViamId		camera;
  viam_filtertype_t	type;
} ViamFilterGetCap;


/* Format filter */
typedef enum ViamFmtFilterType {
    VIAM_FMTFILTER_FORMAT =		VIAM_FILTER_FORMAT
} ViamFmtFilterType;

typedef struct ViamFmtFilter {
  ViamId		filter;
  ViamId		image;
  ViamFmtFilterType	type;
  viam_filtermethod_t	method;
  viam_filterautomode_t	automode;
  viam_filterformat_t	format;
} ViamFmtFilter;

typedef struct ViamUFmtFilter {
  ViamId		filter;
  viam_filtermethod_t	method;
  viam_filterautomode_t	automode;
  viam_filterformat_t	format;
} ViamUFmtFilter;


/* Geometry filters */
typedef enum ViamGeoFilterType {
#ifdef VIAM_OPENCV_SUPPORT
  VIAM_GEOFILTER_DISTORT =		VIAM_FILTER_DISTORT,
  VIAM_GEOFILTER_DISTORT_NN =		VIAM_FILTER_DISTORT_NN,
  VIAM_GEOFILTER_RECTIFY =		VIAM_FILTER_RECTIFY,
  VIAM_GEOFILTER_RECTIFY_NN =		VIAM_FILTER_RECTIFY_NN,
  VIAM_GEOFILTER_SUBSAMPLE =		VIAM_FILTER_SUBSAMPLE,
#endif
  VIAM_GEOFILTER_SCALE =		VIAM_FILTER_SCALE
} ViamGeoFilterType;

typedef struct ViamGeoFilter {
  ViamId		filter;
  ViamId		image;
  ViamGeoFilterType	type;
  viam_filtermethod_t	method;
  viam_filterautomode_t	automode;
  double		left, top, right, bottom;
  double		toWidth, toHeight;
} ViamGeoFilter;

typedef struct ViamUGeoFilter {
  ViamId		filter;
  viam_filtermethod_t	method;
  viam_filterautomode_t	automode;
  double		left, top, right, bottom;
  double		toWidth, toHeight;
} ViamUGeoFilter;


/* Luminance filters */
typedef enum ViamLumFilterType {
  VIAM_LUMFILTER_BRIGHTNESS =		VIAM_FILTER_BRIGHTNESS,
  VIAM_LUMFILTER_EXPOSURE =		VIAM_FILTER_EXPOSURE,
  VIAM_LUMFILTER_SHARPNESS =		VIAM_FILTER_SHARPNESS,
  VIAM_LUMFILTER_GAIN =			VIAM_FILTER_GAIN,
  VIAM_LUMFILTER_SHUTTER =		VIAM_FILTER_SHUTTER
} ViamLumFilterType;

typedef struct ViamLumFilter {
  ViamId		filter;
  ViamId		image;
  ViamLumFilterType	type;
  viam_filtermethod_t	method;
  viam_filterautomode_t	automode;
  double		parameter;
} ViamLumFilter;

typedef struct ViamULumFilter {
  ViamId		filter;
  viam_filtermethod_t	method;
  viam_filterautomode_t	automode;
  double		parameter;
} ViamULumFilter;


/* Color filters */
typedef enum ViamColorFilterType {
  VIAM_COLORFILTER_HUE =		VIAM_FILTER_HUE,
  VIAM_COLORFILTER_SATURATION =		VIAM_FILTER_SATURATION,
  VIAM_COLORFILTER_GAMMA =		VIAM_FILTER_GAMMA,
  VIAM_COLORFILTER_TEMPERATURE =	VIAM_FILTER_TEMPERATURE,
  VIAM_COLORFILTER_WHITE_BALANCE =	VIAM_FILTER_WHITE_BALANCE
} ViamColorFilterType;

typedef struct ViamColorFilter {
  ViamId		filter;
  ViamId		image;
  ViamColorFilterType	type;
  viam_filtermethod_t	method;
  viam_filterautomode_t	automode;
  double		parameter;
} ViamColorFilter;

typedef struct ViamUColorFilter {
  ViamId		filter;
  viam_filtermethod_t	method;
  viam_filterautomode_t	automode;
  double		parameter;
} ViamUColorFilter;


/* Image processing filters */
typedef enum ViamPxFilterType {
#ifdef VIAM_OPENCV_SUPPORT
  VIAM_PXFILTER_LOG =		VIAM_FILTER_LOG
#endif
} ViamPxFilterType;

typedef struct ViamPxFilter {
  ViamId		filter;
  ViamId		image;
  ViamPxFilterType	type;
  viam_filtermethod_t	method;
  viam_filterautomode_t	automode;
  double		parameter;
} ViamPxFilter;

typedef struct ViamUPxFilter {
  ViamId		filter;
  viam_filtermethod_t	method;
  viam_filterautomode_t	automode;
  double		parameter;
} ViamUPxFilter;


/* Miscelaneous filters */
typedef enum ViamMiscFilterType {
  VIAM_MISCFILTER_ZOOM =		VIAM_FILTER_ZOOM,
  VIAM_MISCFILTER_FOCUS =		VIAM_FILTER_FOCUS
} ViamMiscFilterType;

typedef struct ViamMiscFilter {
  ViamId		filter;
  ViamId		image;
  ViamMiscFilterType	type;
  viam_filtermethod_t	method;
  viam_filterautomode_t	automode;
  double		parameter;
} ViamMiscFilter;

typedef struct ViamUMiscFilter {
  ViamId		filter;
  viam_filtermethod_t	method;
  viam_filterautomode_t	automode;
  double		parameter;
} ViamUMiscFilter;


/* --- Image calibration --------------------------------------------- */

typedef struct ViamCalibrate {
  ViamId	bank;		/**< bank name */
  viam_caltype_t type;		/**< calibration type */
  viam_distortionmodel_t dmodel;/**< distortion model */
  int		gridrows;	/**< rows in the calibration grid */
  int		gridcols;	/**< columns in the calibration grid */
  double	gridsize;	/**< dimension of the grid squares */
  double	overlapth;	/**< minimal overlapping distance (pixels) */
  double	motionth;	/**< maximal chessboard motion (pixels) */
  double	precision;	/**< target average reprojection error */
  int		images;		/**< max number of images */
} ViamCalibrate;

typedef struct ViamCalibrationIO {
  ViamId	bank;		/**< bank name */
  ViamIO	op;		/**< I/O operation */
  char		file[1024];/**< file name of saved data */
} ViamCalibrationIO;


/* --- Visual feedback ----------------------------------------------- */

typedef struct ViamDisplay {
  ViamId	bank;		/**< bank name */
  ViamId	image;		/**< image name */
  viam_onoff_t	enable;		/**< control display activation */
  viam_onoff_t	vtag;		/**< control display of performance tags */
  int		width;		/**< window width */
  int		height;		/**< window height */
} ViamDisplay;

typedef struct ViamSave {
  ViamId	bank;		/**< bank name */
  ViamId	image;		/**< image name */
  viam_onoff_t	enable;		/**< control display activation */
  char	pattern[1024];	/**< file name template */
} ViamSave;


/* --- Image poster -------------------------------------------------- */

typedef struct ViamImageHeader {
  ViamId name;		/* camera name */

  unsigned long tacq_sec;	/** time of acquisition (seconds) */
  unsigned long tacq_usec;	/** time of acquisition (microseconds) */
  POM_SENSOR_POS pos;		/**< pom position */

  int nChannels;	/* 1 or 3 channels */
  int depth;		/* pixel depth in bits */
  int width;		/* image width in pixels */
  int height;		/* image height in pixels */
  int imageSize;	/* image size in bytes (image->height*image->widthStep) */
  int widthStep;	/* size of aligned image row in bytes */

  viam_cameracalibration_t calibration;

  unsigned long dataOffset;/* offset of first pixel in image data array below */
  unsigned char data[0];/* data: top-left origin, interleaved color channels */
} ViamImageHeader;

typedef struct ViamImageBank {
  ViamId name;				/* bank name */
  viam_bankcalibration_t calibration;	/* calibration data */
  int nImages;				/* number of images in bank */
  ViamImageHeader image[0];		/* actual size is nImages */
} ViamImageBank;

#endif /* H_VIAMSTRUCT */
